/****************************************************************** 
 *
 * builtinfn.h
 *
 *     Declarations of built-in functions that are available
 *     for Nlv's formulas. ( See 'calc.l' and 'calc.y' )
 *
 *
 * Created: 12 September 1988
 * $Author: mstoufer $
 * $Date: 2000/12/05 19:52:33 $
 * $Id: builtinfn.h,v 1.1.1.1 2000/12/05 19:52:33 mstoufer Exp $
 *
 ****************************************************************** 
 */

#include "structs.h"
#include "calculationclass.h"
#include "graphclass.h"

/* mean */
void Nlv_CalcMean ( Nlv_cc, Nlv_AData * );
void Nlv_ClearMean( Nlv_cc, int );

/* standard deviation */
void Nlv_CalcStddev ( Nlv_cc, Nlv_AData * );
void Nlv_ClearStddev( Nlv_cc, int );

/* latency and throughput */

int    Nlv_InitLatency( Nlv_an anclass );
void   Nlv_CalcLatency(  Nlv_cc calc, Nlv_AData *data );
void   Nlv_ClearLatency( Nlv_cc calc, int num );

int    Nlv_InitThroughput( Nlv_an anclass );
void   Nlv_CalcThroughput(  Nlv_cc calc, Nlv_AData *data );
void   Nlv_ClearThroughput( Nlv_cc calc, int num );

void   Nlv_ShowCalculations( void );

