/****************************************************************** 
 *
 * parseclass.h --
 *
 *     Interface to parsing class. Break raw data into
 *     (name,value) pairs.
 *
 * $Author: mstoufer $
 * $Source: /home/g2/src/projects/NetLogger/src/nlv/parseclass.h,v $
 * $RCSfile: parseclass.h,v $
 * $Revision: 1.1.1.1 $
 * $Locker:  $
 * $Date: 2000/12/05 19:52:32 $
 *
 *
 ****************************************************************** 
 */
#ifndef NLV_PARSE_INCLUDED
#define NLV_PARSE_INCLUDED

#include "errclass.h"
#include "parsedate.h"
#include "structs.h"

/*========= DATA =====================================================*/

#define T Nlv_pa
typedef struct T *T;

/*========= FUNCTIONS ================================================*/

T           Nlv_paNew( );
Nlv_Fields *Nlv_paProcess( T, LogLine l);
void        Nlv_paGetLineTime( LogLine line, double * r );
char       *GetNameAt( T this, int index );
char       *GetValAt ( T this, int index );

#undef T
#endif



