
/*  A Bison parser, made from ../../../src/nlv/nlvky.y
    by GNU Bison version 1.28  */

#define YYBISON 1  /* Identify Bison output.  */

#define yyparse nlvkyparse
#define yylex nlvkylex
#define yyerror nlvkyerror
#define yylval nlvkylval
#define yychar nlvkychar
#define yydebug nlvkydebug
#define yynerrs nlvkynerrs
#define	TITLE	257
#define	NAME	258
#define	QUOTED_STRING	259
#define	FORMULA	260
#define	HOSTALIAS	261
#define	EVENTSET	262
#define	EVENTS	263
#define	TYPE	264
#define	KSTYPE	265
#define	ID	266
#define	HOSTID	267
#define	CALC	268
#define	ANNOTATE	269
#define	MMCHOICE	270
#define	TIMEORD	271
#define	ROTATECL	272
#define	AUTOSCALE	273
#define	CACTION	274
#define	BOOL	275
#define	INT	276
#define	LIST_START	277
#define	LIST_END	278
#define	VAL	279
#define	FLOAT	280
#define	ANALYSIS	281
#define	EPSILON	282
#define	KEYS	283
#define	BRANGE	284
#define	RLIMIT	285
#define	EOFEXIT	286
#define	FIRSTEVENT	287

#line 1 "../../../src/nlv/nlvky.y"

/*
/****************************************************************** 
 *
 * nlvky.y
 *
 *     Yacc grammar for the nlv-keys.cfg file.
 *
 *
 * Author : Dan Gunter
 * Created: September 1998
 *
 * Modification history
 * --------------------
 *
 ****************************************************************** 
 */

/*========= C DECLARATIONS ============================================*/

#include <stdio.h>     /* hello, cruel world */
#include <ctype.h>     /* isalpha, et. al. */
#include <keysclass.h> /* keyset and key manipulation fn's */


#ifndef nlvkylval
#define nlvkylval yylval
#endif

struct ltype;
struct ltype {
   char *data;
   struct ltype *next;
};

/* A buffer for groupaliases */
char aliasbuf[1024];

/* Formula functions */
void write_formula( int ksidx, char* n, char* f );

/* List functions and shortcuts */
void *list_func( char *str, int f );
char *list_to_string( struct ltype *top );
void list_to_array( struct ltype *top, char ***strlist, int *n );
#define add_to_list( str ) do { list_func((str),1); } while(0)
#define clear_list() do { list_func(NULL,0); } while(0)
#define get_list() (list_func(NULL,2))

/*
 * "Object" of type keysclass
 */
extern Nlv_State nlvState;

/*
 * Fill in a keyset structure as we go along 
 */
void keyset_start(void);
void keyset_name(char *);
void keyset_gtype(int);
void keyset_ids( int n, char **strlist );
void keyset_hosts( int n, char **strlist );
void keyset_val_id( char *s, int min, double max );
void keyset_val_di( char *s, double min, int max );
void keyset_val_ii( char *s, int min, int max );
void keyset_val( char *, double min, double max );
void keyset_annotations( int n, char **strlist );
void keyset_timeorder( int n );
void keyset_rotateclients( int n );
void keyset_autoscale( int n );
void keyset_finish(void);
/* 
 * 'wks' is the global variable used in the above routines 
 */
struct keyset_values {
  char          *name;
  int            gtype;
  int            index;
  int            numids;
  char         **ids;
  int            numhosts;
  char         **hosts;
  int            numann;
  char         **ann;
  char          *valField;
  double         vMin, 
                 vMax;
  Nlv_CalcParam *ksCalcParams;
  int            ksCalcParamNum;
  char         **keynames;
  int            keynum;
  int            timeordered;
  int            rotateclients;
  int            autoscale;
} wks;

/*
 * Title of the graph
 * (not keyset-specific)
 */
char *graph_title = NULL;

/*
 * Global cleanup function.
 */
void config_finish( void );

/* 
 * Functions for context-specific token verification.
 * These all return 1 for YES and 0 for NO.
 */
int isvalid_keyset( char *s );
int isvalid_key( char *s );
int isvalid_formname( char *s );
int isvalid_listkey ( char *s );

#define STREQ(a,b) (strcmp(a,b)==0)


#line 123 "../../../src/nlv/nlvky.y"
typedef union {
    int    ival;
    double dval;
    char  *strval;
} YYSTYPE;
#include <stdio.h>

#ifndef __cplusplus
#ifndef __STDC__
#define const
#endif
#endif



#define	YYFINAL		91
#define	YYFLAG		-32768
#define	YYNTBASE	34

#define YYTRANSLATE(x) ((unsigned)(x) <= 287 ? yytranslate[x] : 50)

static const char yytranslate[] = {     0,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     3,     4,     5,     6,
     7,     8,     9,    10,    11,    12,    13,    14,    15,    16,
    17,    18,    19,    20,    21,    22,    23,    24,    25,    26,
    27,    28,    29,    30,    31,    32,    33
};

#if YYDEBUG != 0
static const short yyprhs[] = {     0,
     0,     5,     6,     7,     8,    11,    14,    17,    22,    26,
    28,    30,    32,    34,    36,    38,    42,    44,    48,    50,
    52,    54,    56,    59,    62,    65,    67,    70,    71,    74,
    79,    82,    83,    88,    91,    94,    97,   100,   103,   106,
   109,   112,   117,   122,   127,   132,   136,   140,   144,   151,
   158
};

static const short yyrhs[] = {    35,
    37,    45,    36,     0,     0,     0,     0,    38,    37,     0,
     3,     4,     0,     3,     5,     0,    27,    32,    21,    37,
     0,     7,    39,    40,     0,     4,     0,    22,     0,    26,
     0,     4,     0,    22,     0,    26,     0,    42,    43,    24,
     0,     4,     0,    42,    44,    24,     0,    23,     0,     4,
     0,    22,     0,    26,     0,    43,     4,     0,    43,    22,
     0,    43,    26,     0,     4,     0,    44,     4,     0,     0,
    45,    46,     0,    47,    23,    48,    24,     0,     8,     4,
     0,     0,    48,    23,    49,    24,     0,     9,    41,     0,
    10,    11,     0,    17,    21,     0,    18,    21,     0,    19,
    16,     0,    15,    40,     0,    12,    40,     0,    13,    40,
     0,    25,     4,    26,    26,     0,    25,     4,    26,    22,
     0,    25,     4,    22,    26,     0,    25,     4,    22,    22,
     0,    27,    28,    26,     0,    27,    29,    40,     0,    27,
    30,    40,     0,    27,    31,     4,     4,    26,    26,     0,
    27,    31,     4,     4,     4,     4,     0,    27,    33,     4,
     0
};

#endif

#if YYDEBUG != 0
static const short yyrline[] = { 0,
   167,   170,   173,   176,   177,   178,   181,   185,   188,   202,
   203,   204,   207,   210,   215,   220,   223,   231,   236,   239,
   240,   244,   249,   250,   254,   261,   266,   273,   274,   277,
   280,   290,   291,   294,   296,   297,   298,   299,   300,   310,
   320,   330,   335,   340,   345,   350,   351,   352,   353,   354,
   355
};
#endif


#if YYDEBUG != 0 || defined (YYERROR_VERBOSE)

static const char * const yytname[] = {   "$","error","$undefined.","TITLE",
"NAME","QUOTED_STRING","FORMULA","HOSTALIAS","EVENTSET","EVENTS","TYPE","KSTYPE",
"ID","HOSTID","CALC","ANNOTATE","MMCHOICE","TIMEORD","ROTATECL","AUTOSCALE",
"CACTION","BOOL","INT","LIST_START","LIST_END","VAL","FLOAT","ANALYSIS","EPSILON",
"KEYS","BRANGE","RLIMIT","EOFEXIT","FIRSTEVENT","configfile","init","finish",
"header","hostalias","aliastarget","namelist","keylist","list_start","namevalues",
"keyvalues","body","keyset","set_name","set_options","set_option", NULL
};
#endif

static const short yyr1[] = {     0,
    34,    35,    36,    37,    37,    37,    37,    37,    38,    39,
    39,    39,    40,    40,    40,    40,    41,    41,    42,    43,
    43,    43,    43,    43,    43,    44,    44,    45,    45,    46,
    47,    48,    48,    49,    49,    49,    49,    49,    49,    49,
    49,    49,    49,    49,    49,    49,    49,    49,    49,    49,
    49
};

static const short yyr2[] = {     0,
     4,     0,     0,     0,     2,     2,     2,     4,     3,     1,
     1,     1,     1,     1,     1,     3,     1,     3,     1,     1,
     1,     1,     2,     2,     2,     1,     2,     0,     2,     4,
     2,     0,     4,     2,     2,     2,     2,     2,     2,     2,
     2,     4,     4,     4,     4,     3,     3,     3,     6,     6,
     3
};

static const short yydefact[] = {     2,
     4,     0,     0,     0,    28,     4,     6,     7,    10,    11,
    12,     0,     0,     3,     5,    13,    14,    19,    15,     9,
     0,     4,     0,     1,    29,     0,    20,    21,    22,     0,
     8,    31,    32,    23,    24,    16,    25,     0,     0,    30,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,    17,    34,     0,    35,    40,    41,    39,    36,    37,
    38,     0,     0,     0,     0,     0,     0,    33,    26,     0,
     0,     0,    46,    47,    48,     0,    51,    27,    18,    45,
    44,    43,    42,     0,     0,     0,    50,    49,     0,     0,
     0
};

static const short yydefgoto[] = {    89,
     1,    24,     5,     6,    12,    20,    53,    21,    30,    70,
    14,    25,    26,    38,    51
};

static const short yypact[] = {-32768,
    -3,    54,     5,   -26,-32768,    -3,-32768,-32768,-32768,-32768,
-32768,     3,   -13,    27,-32768,-32768,-32768,-32768,-32768,-32768,
    11,    -3,     7,-32768,-32768,    18,-32768,-32768,-32768,     6,
-32768,-32768,-32768,-32768,-32768,-32768,-32768,    37,    30,-32768,
    15,     9,     3,     3,     3,    41,    42,    28,    60,    23,
    43,-32768,-32768,    61,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,    -9,    40,     3,     3,    64,    65,-32768,-32768,    12,
    -8,    24,-32768,-32768,-32768,    66,-32768,-32768,-32768,-32768,
-32768,-32768,-32768,     8,    67,    46,-32768,-32768,    73,    74,
-32768
};

static const short yypgoto[] = {-32768,
-32768,-32768,    -1,-32768,-32768,   -42,-32768,    34,-32768,-32768,
-32768,-32768,-32768,-32768,-32768
};


#define	YYLAST		75


static const short yytable[] = {     2,
    56,    57,    58,     3,    15,    13,    16,    22,     9,    34,
    32,    85,    71,    80,    27,    78,    72,    81,    52,    55,
    31,    74,    75,     4,    17,    18,    10,    35,    19,    36,
    11,    37,    28,    86,    23,    79,    29,    18,    41,    42,
    33,    43,    44,    61,    45,    82,    46,    47,    48,    83,
    63,    64,    65,    66,    49,    67,    50,     7,     8,    39,
    40,    59,    60,    62,    69,    73,    68,    76,    77,    84,
    87,    88,    90,    91,    54
};

static const short yycheck[] = {     3,
    43,    44,    45,     7,     6,    32,     4,    21,     4,     4,
     4,     4,    22,    22,     4,     4,    26,    26,     4,    11,
    22,    64,    65,    27,    22,    23,    22,    22,    26,    24,
    26,    26,    22,    26,     8,    24,    26,    23,     9,    10,
    23,    12,    13,    16,    15,    22,    17,    18,    19,    26,
    28,    29,    30,    31,    25,    33,    27,     4,     5,    23,
    24,    21,    21,     4,     4,    26,    24,     4,     4,     4,
     4,    26,     0,     0,    41
};
/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
#line 3 "/usr/local/share/bison.simple"
/* This file comes from bison-1.28.  */

/* Skeleton output parser for bison,
   Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/* As a special exception, when this file is copied by Bison into a
   Bison output file, you may use that output file without restriction.
   This special exception was added by the Free Software Foundation
   in version 1.24 of Bison.  */

/* This is the parser code that is written into each bison parser
  when the %semantic_parser declaration is not specified in the grammar.
  It was written by Richard Stallman by simplifying the hairy parser
  used when %semantic_parser is specified.  */

#ifndef YYSTACK_USE_ALLOCA
#ifdef alloca
#define YYSTACK_USE_ALLOCA
#else /* alloca not defined */
#ifdef __GNUC__
#define YYSTACK_USE_ALLOCA
#define alloca __builtin_alloca
#else /* not GNU C.  */
#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi) || (defined (__sun) && defined (__i386))
#define YYSTACK_USE_ALLOCA
#include <alloca.h>
#else /* not sparc */
/* We think this test detects Watcom and Microsoft C.  */
/* This used to test MSDOS, but that is a bad idea
   since that symbol is in the user namespace.  */
#if (defined (_MSDOS) || defined (_MSDOS_)) && !defined (__TURBOC__)
#if 0 /* No need for malloc.h, which pollutes the namespace;
	 instead, just don't use alloca.  */
#include <malloc.h>
#endif
#else /* not MSDOS, or __TURBOC__ */
#if defined(_AIX)
/* I don't know what this was needed for, but it pollutes the namespace.
   So I turned it off.   rms, 2 May 1997.  */
/* #include <malloc.h>  */
 #pragma alloca
#define YYSTACK_USE_ALLOCA
#else /* not MSDOS, or __TURBOC__, or _AIX */
#if 0
#ifdef __hpux /* haible@ilog.fr says this works for HPUX 9.05 and up,
		 and on HPUX 10.  Eventually we can turn this on.  */
#define YYSTACK_USE_ALLOCA
#define alloca __builtin_alloca
#endif /* __hpux */
#endif
#endif /* not _AIX */
#endif /* not MSDOS, or __TURBOC__ */
#endif /* not sparc */
#endif /* not GNU C */
#endif /* alloca not defined */
#endif /* YYSTACK_USE_ALLOCA not defined */

#ifdef YYSTACK_USE_ALLOCA
#define YYSTACK_ALLOC alloca
#else
#define YYSTACK_ALLOC malloc
#endif

/* Note: there must be only one dollar sign in this file.
   It is replaced by the list of actions, each action
   as one case of the switch.  */

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYACCEPT	goto yyacceptlab
#define YYABORT 	goto yyabortlab
#define YYERROR		goto yyerrlab1
/* Like YYERROR except do call yyerror.
   This remains here temporarily to ease the
   transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */
#define YYFAIL		goto yyerrlab
#define YYRECOVERING()  (!!yyerrstatus)
#define YYBACKUP(token, value) \
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    { yychar = (token), yylval = (value);			\
      yychar1 = YYTRANSLATE (yychar);				\
      YYPOPSTACK;						\
      goto yybackup;						\
    }								\
  else								\
    { yyerror ("syntax error: cannot back up"); YYERROR; }	\
while (0)

#define YYTERROR	1
#define YYERRCODE	256

#ifndef YYPURE
#define YYLEX		yylex()
#endif

#ifdef YYPURE
#ifdef YYLSP_NEEDED
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, &yylloc, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval, &yylloc)
#endif
#else /* not YYLSP_NEEDED */
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval)
#endif
#endif /* not YYLSP_NEEDED */
#endif

/* If nonreentrant, generate the variables here */

#ifndef YYPURE

int	yychar;			/*  the lookahead symbol		*/
YYSTYPE	yylval;			/*  the semantic value of the		*/
				/*  lookahead symbol			*/

#ifdef YYLSP_NEEDED
YYLTYPE yylloc;			/*  location data for the lookahead	*/
				/*  symbol				*/
#endif

int yynerrs;			/*  number of parse errors so far       */
#endif  /* not YYPURE */

#if YYDEBUG != 0
int yydebug;			/*  nonzero means print parse trace	*/
/* Since this is uninitialized, it does not stop multiple parsers
   from coexisting.  */
#endif

/*  YYINITDEPTH indicates the initial size of the parser's stacks	*/

#ifndef	YYINITDEPTH
#define YYINITDEPTH 200
#endif

/*  YYMAXDEPTH is the maximum size the stacks can grow to
    (effective only if the built-in stack extension method is used).  */

#if YYMAXDEPTH == 0
#undef YYMAXDEPTH
#endif

#ifndef YYMAXDEPTH
#define YYMAXDEPTH 10000
#endif

/* Define __yy_memcpy.  Note that the size argument
   should be passed with type unsigned int, because that is what the non-GCC
   definitions require.  With GCC, __builtin_memcpy takes an arg
   of type size_t, but it can handle unsigned int.  */

#if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
#define __yy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
#else				/* not GNU C or C++ */
#ifndef __cplusplus

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (to, from, count)
     char *to;
     char *from;
     unsigned int count;
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#else /* __cplusplus */

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (char *to, char *from, unsigned int count)
{
  register char *t = to;
  register char *f = from;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#endif
#endif

#line 217 "/usr/local/share/bison.simple"

/* The user can define YYPARSE_PARAM as the name of an argument to be passed
   into yyparse.  The argument should have type void *.
   It should actually point to an object.
   Grammar actions can access the variable by casting it
   to the proper pointer type.  */

#ifdef YYPARSE_PARAM
#ifdef __cplusplus
#define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else /* not __cplusplus */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
#endif /* not __cplusplus */
#else /* not YYPARSE_PARAM */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif /* not YYPARSE_PARAM */

/* Prevent warning if -Wstrict-prototypes.  */
#ifdef __GNUC__
#ifdef YYPARSE_PARAM
int yyparse (void *);
#else
int yyparse (void);
#endif
#endif

int
yyparse(YYPARSE_PARAM_ARG)
     YYPARSE_PARAM_DECL
{
  register int yystate;
  register int yyn;
  register short *yyssp;
  register YYSTYPE *yyvsp;
  int yyerrstatus;	/*  number of tokens to shift before error messages enabled */
  int yychar1 = 0;		/*  lookahead token as an internal (translated) token number */

  short	yyssa[YYINITDEPTH];	/*  the state stack			*/
  YYSTYPE yyvsa[YYINITDEPTH];	/*  the semantic value stack		*/

  short *yyss = yyssa;		/*  refer to the stacks thru separate pointers */
  YYSTYPE *yyvs = yyvsa;	/*  to allow yyoverflow to reallocate them elsewhere */

#ifdef YYLSP_NEEDED
  YYLTYPE yylsa[YYINITDEPTH];	/*  the location stack			*/
  YYLTYPE *yyls = yylsa;
  YYLTYPE *yylsp;

#define YYPOPSTACK   (yyvsp--, yyssp--, yylsp--)
#else
#define YYPOPSTACK   (yyvsp--, yyssp--)
#endif

  int yystacksize = YYINITDEPTH;
  int yyfree_stacks = 0;

#ifdef YYPURE
  int yychar;
  YYSTYPE yylval;
  int yynerrs;
#ifdef YYLSP_NEEDED
  YYLTYPE yylloc;
#endif
#endif

  YYSTYPE yyval;		/*  the variable used to return		*/
				/*  semantic values from the action	*/
				/*  routines				*/

  int yylen;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Starting parse\n");
#endif

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */

  yyssp = yyss - 1;
  yyvsp = yyvs;
#ifdef YYLSP_NEEDED
  yylsp = yyls;
#endif

/* Push a new state, which is found in  yystate  .  */
/* In all cases, when you get here, the value and location stacks
   have just been pushed. so pushing a state here evens the stacks.  */
yynewstate:

  *++yyssp = yystate;

  if (yyssp >= yyss + yystacksize - 1)
    {
      /* Give user a chance to reallocate the stack */
      /* Use copies of these so that the &'s don't force the real ones into memory. */
      YYSTYPE *yyvs1 = yyvs;
      short *yyss1 = yyss;
#ifdef YYLSP_NEEDED
      YYLTYPE *yyls1 = yyls;
#endif

      /* Get the current used size of the three stacks, in elements.  */
      int size = yyssp - yyss + 1;

#ifdef yyoverflow
      /* Each stack pointer address is followed by the size of
	 the data in use in that stack, in bytes.  */
#ifdef YYLSP_NEEDED
      /* This used to be a conditional around just the two extra args,
	 but that might be undefined if yyoverflow is a macro.  */
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yyls1, size * sizeof (*yylsp),
		 &yystacksize);
#else
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yystacksize);
#endif

      yyss = yyss1; yyvs = yyvs1;
#ifdef YYLSP_NEEDED
      yyls = yyls1;
#endif
#else /* no yyoverflow */
      /* Extend the stack our own way.  */
      if (yystacksize >= YYMAXDEPTH)
	{
	  yyerror("parser stack overflow");
	  if (yyfree_stacks)
	    {
	      free (yyss);
	      free (yyvs);
#ifdef YYLSP_NEEDED
	      free (yyls);
#endif
	    }
	  return 2;
	}
      yystacksize *= 2;
      if (yystacksize > YYMAXDEPTH)
	yystacksize = YYMAXDEPTH;
#ifndef YYSTACK_USE_ALLOCA
      yyfree_stacks = 1;
#endif
      yyss = (short *) YYSTACK_ALLOC (yystacksize * sizeof (*yyssp));
      __yy_memcpy ((char *)yyss, (char *)yyss1,
		   size * (unsigned int) sizeof (*yyssp));
      yyvs = (YYSTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yyvsp));
      __yy_memcpy ((char *)yyvs, (char *)yyvs1,
		   size * (unsigned int) sizeof (*yyvsp));
#ifdef YYLSP_NEEDED
      yyls = (YYLTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yylsp));
      __yy_memcpy ((char *)yyls, (char *)yyls1,
		   size * (unsigned int) sizeof (*yylsp));
#endif
#endif /* no yyoverflow */

      yyssp = yyss + size - 1;
      yyvsp = yyvs + size - 1;
#ifdef YYLSP_NEEDED
      yylsp = yyls + size - 1;
#endif

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Stack size increased to %d\n", yystacksize);
#endif

      if (yyssp >= yyss + yystacksize - 1)
	YYABORT;
    }

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Entering state %d\n", yystate);
#endif

  goto yybackup;
 yybackup:

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
/* yyresume: */

  /* First try to decide what to do without reference to lookahead token.  */

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* yychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (yychar == YYEMPTY)
    {
#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Reading a token: ");
#endif
      yychar = YYLEX;
    }

  /* Convert token to internal form (in yychar1) for indexing tables with */

  if (yychar <= 0)		/* This means end of input. */
    {
      yychar1 = 0;
      yychar = YYEOF;		/* Don't call YYLEX any more */

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Now at end of input.\n");
#endif
    }
  else
    {
      yychar1 = YYTRANSLATE(yychar);

#if YYDEBUG != 0
      if (yydebug)
	{
	  fprintf (stderr, "Next token is %d (%s", yychar, yytname[yychar1]);
	  /* Give the individual parser a way to print the precise meaning
	     of a token, for further debugging info.  */
#ifdef YYPRINT
	  YYPRINT (stderr, yychar, yylval);
#endif
	  fprintf (stderr, ")\n");
	}
#endif
    }

  yyn += yychar1;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != yychar1)
    goto yydefault;

  yyn = yytable[yyn];

  /* yyn is what to do for this token type in this state.
     Negative => reduce, -yyn is rule number.
     Positive => shift, yyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrlab;

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting token %d (%s), ", yychar, yytname[yychar1]);
#endif

  /* Discard the token being shifted unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  /* count tokens shifted since error; after three, turn off error status.  */
  if (yyerrstatus) yyerrstatus--;

  yystate = yyn;
  goto yynewstate;

/* Do the default action for the current state.  */
yydefault:

  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;

/* Do a reduction.  yyn is the number of a rule to reduce with.  */
yyreduce:
  yylen = yyr2[yyn];
  if (yylen > 0)
    yyval = yyvsp[1-yylen]; /* implement default value of the action */

#if YYDEBUG != 0
  if (yydebug)
    {
      int i;

      fprintf (stderr, "Reducing via rule %d (line %d), ",
	       yyn, yyrline[yyn]);

      /* Print the symbols being reduced, and their result.  */
      for (i = yyprhs[yyn]; yyrhs[i] > 0; i++)
	fprintf (stderr, "%s ", yytname[yyrhs[i]]);
      fprintf (stderr, " -> %s\n", yytname[yyr1[yyn]]);
    }
#endif


  switch (yyn) {

case 2:
#line 170 "../../../src/nlv/nlvky.y"
{ wks.index = -1; ;
    break;}
case 3:
#line 173 "../../../src/nlv/nlvky.y"
{ config_finish(); ;
    break;}
case 6:
#line 178 "../../../src/nlv/nlvky.y"
{ if ( graph_title ) free(graph_title); 
                     graph_title = strdup( yyvsp[0].strval );
                   ;
    break;}
case 7:
#line 181 "../../../src/nlv/nlvky.y"
{ if (graph_title) free(graph_title);
                     yyvsp[0].strval[strlen(yyvsp[0].strval)-1] = '\0';
                     graph_title = strdup( ++yyvsp[0].strval );
                   ;
    break;}
case 8:
#line 185 "../../../src/nlv/nlvky.y"
{ ; ;
    break;}
case 9:
#line 188 "../../../src/nlv/nlvky.y"
{
  struct ltype *t;
  int    n;
  char **strlist;

  /* get a pointer to top of llist */
  t = get_list();
  /* make our list into an array of strings */
  list_to_array( t, &strlist, &n );
  /* add this to the hostalias record in the keysclass "object" */
  Nlv_kyAddHostalias( nlvState.ky, n, strlist, aliasbuf );
;
    break;}
case 10:
#line 202 "../../../src/nlv/nlvky.y"
{ strcpy(aliasbuf,yyvsp[0].strval); ;
    break;}
case 11:
#line 203 "../../../src/nlv/nlvky.y"
{ sprintf(aliasbuf,"%d",yyvsp[0].ival); ;
    break;}
case 12:
#line 204 "../../../src/nlv/nlvky.y"
{ sprintf(aliasbuf,"%f",yyvsp[0].dval); ;
    break;}
case 13:
#line 207 "../../../src/nlv/nlvky.y"
{ clear_list(); 
                          add_to_list( yyvsp[0].strval ); 
                         ;
    break;}
case 14:
#line 210 "../../../src/nlv/nlvky.y"
{ char *str = (char *)malloc(128); 
	        clear_list();
		sprintf( str, "%d", yyvsp[0].ival );
	        add_to_list( str );
	      ;
    break;}
case 15:
#line 215 "../../../src/nlv/nlvky.y"
{ char *str = (char *)malloc(128); 
	        clear_list();
		sprintf( str, "%lf", yyvsp[0].dval );
	        add_to_list( str );
	      ;
    break;}
case 17:
#line 223 "../../../src/nlv/nlvky.y"
{ 
                clear_list();
                if ( isvalid_key( yyvsp[0].strval ) )
                  add_to_list( yyvsp[0].strval );
                else
                  nlvkyerror("Invalid key");
	        list_to_array( get_list(), &wks.keynames, &wks.keynum );
               ;
    break;}
case 18:
#line 231 "../../../src/nlv/nlvky.y"
{ 
                list_to_array( get_list(), &wks.keynames, &wks.keynum );
               ;
    break;}
case 19:
#line 236 "../../../src/nlv/nlvky.y"
{ clear_list(); ;
    break;}
case 20:
#line 239 "../../../src/nlv/nlvky.y"
{ add_to_list( yyvsp[0].strval ); ;
    break;}
case 21:
#line 240 "../../../src/nlv/nlvky.y"
{ char *str = (char *)malloc(128); 
		  sprintf( str, "%d", yyvsp[0].ival );
	          add_to_list( str );
	         ;
    break;}
case 22:
#line 244 "../../../src/nlv/nlvky.y"
{ char *str = (char *)malloc(128); 
	        clear_list();
		sprintf( str, "%lf", yyvsp[0].dval );
	        add_to_list( str );
	      ;
    break;}
case 23:
#line 249 "../../../src/nlv/nlvky.y"
{ add_to_list( yyvsp[0].strval ); ;
    break;}
case 24:
#line 250 "../../../src/nlv/nlvky.y"
{ char *str = (char *)malloc(128); 
		           sprintf( str, "%d", yyvsp[0].ival );
	                   add_to_list( str );
	                 ;
    break;}
case 25:
#line 254 "../../../src/nlv/nlvky.y"
{ char *str = (char *)malloc(128); 
	        clear_list();
		sprintf( str, "%lf", yyvsp[0].dval );
	        add_to_list( str );
	      ;
    break;}
case 26:
#line 261 "../../../src/nlv/nlvky.y"
{ if ( isvalid_key( yyvsp[0].strval ) )
                             add_to_list( yyvsp[0].strval );
                           else
                             nlvkyerror("Invalid key"); 
                          ;
    break;}
case 27:
#line 266 "../../../src/nlv/nlvky.y"
{ if ( isvalid_key( yyvsp[0].strval ))
                              add_to_list( yyvsp[0].strval );
                            else
                              nlvkyerror("Invalid key");
                          ;
    break;}
case 30:
#line 277 "../../../src/nlv/nlvky.y"
{ keyset_finish(); ;
    break;}
case 31:
#line 280 "../../../src/nlv/nlvky.y"
{
  if (isvalid_keyset(yyvsp[0].strval)) {
    keyset_start();
    keyset_name( yyvsp[0].strval );
  }
  else
    nlvkyerror("Invalid keyset name");
;
    break;}
case 34:
#line 295 "../../../src/nlv/nlvky.y"
{ ; ;
    break;}
case 35:
#line 296 "../../../src/nlv/nlvky.y"
{ keyset_gtype( yyvsp[0].ival ); ;
    break;}
case 36:
#line 297 "../../../src/nlv/nlvky.y"
{ keyset_timeorder( yyvsp[0].ival ); ;
    break;}
case 37:
#line 298 "../../../src/nlv/nlvky.y"
{ keyset_rotateclients( yyvsp[0].ival ); ;
    break;}
case 38:
#line 299 "../../../src/nlv/nlvky.y"
{ keyset_autoscale( yyvsp[0].ival ); ;
    break;}
case 39:
#line 300 "../../../src/nlv/nlvky.y"
{
	               struct ltype *t;
		       /* make sure all values are at least legal keys */
                       for ( t = get_list(); t; t = t->next )
			 if ( !isvalid_listkey(t->data) )
			   nlvkyerror("*Invalid ANNOTATION key");
		       /* put linked list values into keyset structure */
		       list_to_array( get_list(), &(wks.ann), &(wks.numann) );
		       keyset_annotations( wks.numann, wks.ann );
                     ;
    break;}
case 40:
#line 310 "../../../src/nlv/nlvky.y"
{
	               struct ltype *t;
		       /* make sure all values are at least legal keys */
                       for ( t = get_list(); t; t = t->next )
			 if ( !isvalid_listkey(t->data) )
			   nlvkyerror("*Invalid ID key");
		       /* put linked list values into keyset structure */
		       list_to_array( get_list(), &(wks.ids), &(wks.numids) );
		       keyset_ids( wks.numids, wks.ids );
                     ;
    break;}
case 41:
#line 320 "../../../src/nlv/nlvky.y"
{
	                 struct ltype *t;
			 /* make sure all values are at least legal keys */
                         for ( t = get_list(); t; t = t->next )
			   if ( !isvalid_listkey(t->data) )
			     nlvkyerror("*Invalid GROUP key");
			 /* put linked list values into keyset structure */
		         list_to_array( get_list(), &(wks.hosts), &(wks.numhosts) );
		         keyset_hosts( wks.numhosts, wks.hosts );
                         ;
    break;}
case 42:
#line 330 "../../../src/nlv/nlvky.y"
{ if ( isvalid_listkey( yyvsp[-2].strval ))
                                   keyset_val( yyvsp[-2].strval, yyvsp[-1].dval, yyvsp[0].dval );
                                 else
                                   nlvkyerror("Invalid key for 'val'");
                              ;
    break;}
case 43:
#line 335 "../../../src/nlv/nlvky.y"
{ if ( isvalid_listkey( yyvsp[-2].strval ))
                                   keyset_val_di( yyvsp[-2].strval, yyvsp[-1].dval, yyvsp[0].ival );
                                 else
                                   nlvkyerror("Invalid key for 'val'");
                              ;
    break;}
case 44:
#line 340 "../../../src/nlv/nlvky.y"
{ if ( isvalid_listkey( yyvsp[-2].strval ))
                                   keyset_val_id( yyvsp[-2].strval, yyvsp[-1].ival, yyvsp[0].dval );
                                 else
                                   nlvkyerror("Invalid key for 'val'");
                              ;
    break;}
case 45:
#line 345 "../../../src/nlv/nlvky.y"
{ if ( isvalid_listkey( yyvsp[-2].strval ))
                                   keyset_val_ii( yyvsp[-2].strval, yyvsp[-1].ival, yyvsp[0].ival );
                                 else
                                   nlvkyerror("Invalid key for 'val'");
                              ;
    break;}
case 46:
#line 350 "../../../src/nlv/nlvky.y"
{ ; ;
    break;}
case 47:
#line 351 "../../../src/nlv/nlvky.y"
{ ; ;
    break;}
case 48:
#line 352 "../../../src/nlv/nlvky.y"
{ ; ;
    break;}
case 49:
#line 353 "../../../src/nlv/nlvky.y"
{ ; ;
    break;}
case 50:
#line 354 "../../../src/nlv/nlvky.y"
{ ; ;
    break;}
case 51:
#line 355 "../../../src/nlv/nlvky.y"
{ ; ;
    break;}
}
   /* the action file gets copied in in place of this dollarsign */
#line 543 "/usr/local/share/bison.simple"

  yyvsp -= yylen;
  yyssp -= yylen;
#ifdef YYLSP_NEEDED
  yylsp -= yylen;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

  *++yyvsp = yyval;

#ifdef YYLSP_NEEDED
  yylsp++;
  if (yylen == 0)
    {
      yylsp->first_line = yylloc.first_line;
      yylsp->first_column = yylloc.first_column;
      yylsp->last_line = (yylsp-1)->last_line;
      yylsp->last_column = (yylsp-1)->last_column;
      yylsp->text = 0;
    }
  else
    {
      yylsp->last_line = (yylsp+yylen-1)->last_line;
      yylsp->last_column = (yylsp+yylen-1)->last_column;
    }
#endif

  /* Now "shift" the result of the reduction.
     Determine what state that goes to,
     based on the state we popped back to
     and the rule number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTBASE] + *yyssp;
  if (yystate >= 0 && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTBASE];

  goto yynewstate;

yyerrlab:   /* here on detecting error */

  if (! yyerrstatus)
    /* If not already recovering from an error, report this error.  */
    {
      ++yynerrs;

#ifdef YYERROR_VERBOSE
      yyn = yypact[yystate];

      if (yyn > YYFLAG && yyn < YYLAST)
	{
	  int size = 0;
	  char *msg;
	  int x, count;

	  count = 0;
	  /* Start X at -yyn if nec to avoid negative indexes in yycheck.  */
	  for (x = (yyn < 0 ? -yyn : 0);
	       x < (sizeof(yytname) / sizeof(char *)); x++)
	    if (yycheck[x + yyn] == x)
	      size += strlen(yytname[x]) + 15, count++;
	  msg = (char *) malloc(size + 15);
	  if (msg != 0)
	    {
	      strcpy(msg, "parse error");

	      if (count < 5)
		{
		  count = 0;
		  for (x = (yyn < 0 ? -yyn : 0);
		       x < (sizeof(yytname) / sizeof(char *)); x++)
		    if (yycheck[x + yyn] == x)
		      {
			strcat(msg, count == 0 ? ", expecting `" : " or `");
			strcat(msg, yytname[x]);
			strcat(msg, "'");
			count++;
		      }
		}
	      yyerror(msg);
	      free(msg);
	    }
	  else
	    yyerror ("parse error; also virtual memory exceeded");
	}
      else
#endif /* YYERROR_VERBOSE */
	yyerror("parse error");
    }

  goto yyerrlab1;
yyerrlab1:   /* here on error raised explicitly by an action */

  if (yyerrstatus == 3)
    {
      /* if just tried and failed to reuse lookahead token after an error, discard it.  */

      /* return failure if at end of input */
      if (yychar == YYEOF)
	YYABORT;

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Discarding token %d (%s).\n", yychar, yytname[yychar1]);
#endif

      yychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token
     after shifting the error token.  */

  yyerrstatus = 3;		/* Each real token shifted decrements this */

  goto yyerrhandle;

yyerrdefault:  /* current state does not do anything special for the error token. */

#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */
  yyn = yydefact[yystate];  /* If its default is to accept any token, ok.  Otherwise pop it.*/
  if (yyn) goto yydefault;
#endif

yyerrpop:   /* pop the current state because it cannot handle the error token */

  if (yyssp == yyss) YYABORT;
  yyvsp--;
  yystate = *--yyssp;
#ifdef YYLSP_NEEDED
  yylsp--;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "Error: state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

yyerrhandle:

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yyerrdefault;

  yyn += YYTERROR;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != YYTERROR)
    goto yyerrdefault;

  yyn = yytable[yyn];
  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrpop;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrpop;

  if (yyn == YYFINAL)
    YYACCEPT;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting error token, ");
#endif

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  yystate = yyn;
  goto yynewstate;

 yyacceptlab:
  /* YYACCEPT comes here.  */
  if (yyfree_stacks)
    {
      free (yyss);
      free (yyvs);
#ifdef YYLSP_NEEDED
      free (yyls);
#endif
    }
  return 0;

 yyabortlab:
  /* YYABORT comes here.  */
  if (yyfree_stacks)
    {
      free (yyss);
      free (yyvs);
#ifdef YYLSP_NEEDED
      free (yyls);
#endif
    }
  return 1;
}
#line 358 "../../../src/nlv/nlvky.y"



/*========= C IMPLEMENTATIONS =========================================*/

/*
 *----------------------------------------------------------------------
 *
 * write_formula -- Write the user-defined formula to a temporary file.
 *
 * Calls:
 *
 * Algorithm:
 *
 *----------------------------------------------------------------------
 */
void write_formula( int ksidx, char* n, char* f )
{
  printf("keyset=%d name=%s formula=%s\n", ksidx, n, f );
  free(n);
  free(f);
}

/*
 *----------------------------------------------------------------------
 *
 * list_func -- Clear, add to, and give access to a linked list.
 *
 * Calls: malloc, free, strcpy
 *
 * Algorithm: The top of the list is held in a static variable,
 *            and the appropriate list operation is chosen by
 *            an integer argument.
 *
 *----------------------------------------------------------------------
 */


void *list_func( char *str, int f )
{
   static struct ltype *top;
   struct ltype *p, *ins, *newnode;

   switch( f ) {
     /* clear */
   case 0:
     for ( p=top; top; p=top ) {
       top = top->next;
       if ( p->data ) free( p->data );
       free( p );
     }
     top = NULL;
     return NULL;
     /* add */
   case 1:
     if ( !str )
       fprintf(stderr,"Warning: Empty hostalias list in config file\n");
     else {
       for ( ins=NULL, p=top; p; ins=p, p=p->next );
       newnode = (struct ltype *)malloc( sizeof(struct ltype));
       newnode->data = (char *)malloc((strlen(str)+1)*sizeof(char));
       strcpy( newnode->data, str );
       newnode->next = NULL;
       if ( ins )
	 ins->next = newnode;
       else
	 top = newnode;
     }
     return NULL;
     /* get */
   case 2:
     return top;
   }
}

/*
 *----------------------------------------------------------------------
 *
 * list_to_string -- Transform a linked list of strings to a string.
 *
 * Calls: realloc, calloc, strlen, strcat
 *
 * Algorithm: Effortlessly realloced.
 *
 *----------------------------------------------------------------------
 */

char *list_to_string( struct ltype *top )
{
  char *r;
  struct ltype *p;

  for ( r=NULL, p=top; p; p = p->next ) {
    if ( p->data ) {
      if ( r )
	r = (char *)realloc( r, (strlen(p->data)+1) * sizeof(char) );
      else
	r = (char *)calloc( strlen(p->data)+1, sizeof(char) );
      strcat( r, p->data );
    }
  }
  return r;
}

/*
 *----------------------------------------------------------------------
 *
 * list_to_array -- Transform linked list to array.
 *                  Array strings are a COPY of list strings.
 *
 * Calls: calloc, strdup
 *
 * Algorithm: find list length
 *            allocate that many slots in array
 *            traverse the list and COPY strings
 *
 *----------------------------------------------------------------------
 */

void list_to_array( struct ltype *top, char ***strlist, int *n )
{
  struct ltype *p;
  int len;

  /* calculate list length */
  for ( len=0, p=top; p; p=p->next, len++ );
  /* allocate space for each character pointer */
  *strlist = ( char ** )malloc( len * sizeof(char *));
  /* copy strings from linked list to array */
  for ( p=top, *n=0; p; p=p->next, (*n)++ )
    (*strlist)[*n] = strdup( p->data );
}

/*
 *----------------------------------------------------------------------
 *
 * isvalid_keyset -- Check if token is valid as a keyset name.
 *
 * Calls:
 *
 * Algorithm: String must be non-empty and at least of length 2
 *            1st character must be a '+' or '-'
 *            2nd character must be alphabetic
 *            subsequent characters must be alphanumeric or '_'
 *
 *----------------------------------------------------------------------
 */
int isvalid_keyset ( char *s )
{
  int valid = 1; /* assume valid */

  /* non-empty of length at least 2 */
  if ( !s || strlen(s) < 2 )
    valid = 0;
  else {
    /* first character must be '+' or '-' */
    if ( *s != '+' && *s != '-' )
      valid = 0;
    else {
      s++;
       /* second character must be alpha */
      if ( !isalpha( *s ) )
	valid = 0;
      else {
	s++;
	/* subsequent characters must be alphanumeric or '_' */
	for ( ; valid && *s; s++ )
	  if ( *s != '_' && !isalnum(*s) )
	    valid = 0;
      }
    }
  }
  return valid;
}

/*
 *----------------------------------------------------------------------
 *
 * isvalid_listkey --
 *
 * Calls:
 *
 * Algorithm: String must be non-empty and at least of length 1
 *            All characters must be alphanumeric or '_' or '.'
 *
 *----------------------------------------------------------------------
 */
int isvalid_listkey ( char *s )
{
  int valid = 1; /* assume valid */

  /* non-empty of length at least 1 */
  if ( !s || strlen(s) < 1 )
    valid = 0;
  else {
    /* subsequent characters must be alphanumeric or '_' */
    for ( ; valid && *s; s++ )
      if ( *s != '_' && *s != '.' && !isalnum(*s) )
	valid = 0;
  }
  return valid;
}

/*
 *----------------------------------------------------------------------
 *
 * isvalid_key -- 
 *
 * Calls:
 *
 * Algorithm:
 *
 *----------------------------------------------------------------------
 */
int isvalid_key( char *s )
{
  if ( isvalid_keyset( s ) ) {
    /* check for duplicate keys */
    struct ltype *p;
    char buf[ 128 ];
    for ( p=get_list(); p ; p=p->next ) {
      if ( strcmp( (s+1), (p->data+1)) == 0 ) {
	sprintf(buf,"*Duplicate key '%s'",(s+1));
	nlvkyerror(buf);
      }
    }
    return 1;
  }
  return 0;
}

/*
 *----------------------------------------------------------------------
 *
 * isvalid_formname -- Check if token is valid as a formula name.
 *
 * Calls:
 *
 * Algorithm: of length at least 1
 *            first character must be alpha
 *            subsequent characters may be alpha or '_'
 *
 *----------------------------------------------------------------------
 */
int isvalid_formname( char *s )
{
  int valid = 1; /* assume valid */

  if ( !s || !(*s) )
    valid = 0;
  else {
    if ( !isalpha(*s) )
      valid = 0;
    else {
      for ( ; valid && *s; s++ )
	if ( !isalnum(*s) )
	  valid = 0;
    }
  }

  return valid;
}

/*
 *----------------------------------------------------------------------
 *
 * keyset_start      |
 * keyset_finish     |
 * keyset_name       |
 * keyset_gtype      |
 * keyset_ids        |
 * keyset_hosts      +-- setters for keyset values
 * keyset_val        |
 * keyset_autoscale  |
 * keyset_timeorder  |
 *
 * Calls:
 *
 * Algorithm: Use global variable 'wks' (working keyset).
 *
 *----------------------------------------------------------------------
 */
void keyset_start(void)
{
  int i;

  /* increment keyset index and clear struct */
  i = wks.index;
  memset( &wks, 0, sizeof(wks) );
  wks.index  = i + 1;

  /* reset key number to 0 */
  wks.keynum = 0;

  /* Put invalid value in timeordered */
  wks.timeordered = -1;
  /* Put invalid value in rotateclients */
  wks.rotateclients = -1;
  /* Put invalid value in autoscale */
  wks.autoscale = -2;
}

void keyset_name( char *s )
{
  wks.name = s;
}
void keyset_gtype( int t )
{
  wks.gtype = t;
}
void keyset_ids( int n, char **strlist )
{
  wks.numids = n;
  wks.ids = strlist;
}
void keyset_hosts( int n, char **strlist )
{
  wks.numhosts = n;
  wks.hosts    = strlist;
}
void keyset_annotations( int n, char **strlist )
{
  wks.numann = n;
  wks.ann = strlist;
}
void keyset_timeorder( int n )
{
   wks.timeordered = n;
}
void keyset_rotateclients( int n )
{
   wks.rotateclients = n;
}
void keyset_autoscale( int n )
{
  wks.autoscale = n;
}

void keyset_val_id( char *s, int min, double max ) 
{ 
  keyset_val( s, (double) min, max ); 
}

void keyset_val_di( char *s, double min, int max )
{ 
  keyset_val( s, min, (double)max );
}

void keyset_val_ii( char *s, int min, int max )
{ 
  keyset_val( s, (double)min, (double)max ); 
} 

void keyset_val( char *s, double min, double max )
{
  wks.valField = s;
  wks.vMin = min;
  wks.vMax = max;
}

void keyset_finish(void)
{
  Nlv_KeySet *ksPtr;
  int i;

  /* default fields */
  if ( wks.numids < 1 ) {
    wks.numids = 1;
    wks.ids = (char **)malloc(sizeof(char*));
    *wks.ids   = strdup("HOST");
  }
  if ( wks.numhosts < 1 ) {
    wks.numhosts = 1;
    wks.hosts = (char **)malloc(sizeof(char*));
    *wks.hosts   = strdup("HOST");
  }
  if ( wks.numann < 1 ) {
    wks.numann = 3;
    wks.ann = (char **)malloc(3*sizeof(char *));
    wks.ann[0] = strdup("TIME");
    wks.ann[1] = strdup("HOST");
    wks.ann[2] = strdup("PROG");
  }
  if ( wks.timeordered < 0 ) {
    wks.timeordered = 0;
  }
  if ( wks.rotateclients < 0 ) {
    wks.rotateclients = 0;
  }
  if ( wks.autoscale == -2 ) {
    wks.autoscale = 0;
  }

  /***
   * For debugging..
   * dumpkeys();
   **/

  ksPtr = Nlv_kyAddKeySet( nlvState.ky,
			   wks.name,
			   wks.gtype,
			   wks.index,
			   wks.numids,
			   wks.ids,
			   wks.numhosts,
			   wks.hosts,
			   wks.numann,
			   wks.ann,
			   wks.valField,
			   wks.vMin,
			   wks.vMax,
			   wks.ksCalcParams,
			   wks.ksCalcParamNum,
			   wks.timeordered,
			   wks.rotateclients,
			   wks.autoscale );
  if ( ksPtr == NULL )
    nlvkyerror("Unable to add keyset");
  for ( i=0; i < wks.keynum; i++ ) {
    Nlv_kyAddKey( nlvState.ky, ksPtr, wks.keynames[i] );
  }
}

/*
 * Do any cleanup/finalizations when file parsing
 * is done.
 */
void config_finish()
{
  Nlv_kySetTitle( nlvState.ky, graph_title );
}

dumpkeys() 
{
  int i;
  /* some debugging */
  fprintf(stderr,"(debug) keyset '%s':\n", wks.name);
  fprintf(stderr,"\tevents={");
  for ( i=0; i < wks.keynum; i++ ) fprintf(stderr,"%s ",wks.keynames[i]);
  fprintf(stderr,"}\n\tids={");
  for(i=0;i<wks.numids;i++) fprintf(stderr,"%s ", wks.ids[i]);
  fprintf(stderr,"}\n\tgroups={");
  for(i=0;i<wks.numhosts;i++) fprintf(stderr,"%s ", wks.hosts[i]);
  fprintf(stderr,"}\n");
}
