/****************************************************************** 
 *
 * analysisclass.h --
 *
 *     Interface to analysis class.
 *
 *
 * Created: 9 June 1998
 * $Author: mstoufer $
 * $Date: 2000/12/05 19:52:33 $
 * $Id: analysisclass.h,v 1.1.1.1 2000/12/05 19:52:33 mstoufer Exp $
 *
 ****************************************************************** 
 */
#ifndef NLV_ANALYSIS_INCLUDED
#define NLV_ANALYSIS_INCLUDED

#include "structs.h"
#include "calculationclass.h"

/*========= DATA =====================================================*/

#define T Nlv_an
typedef struct T *T;

/*========= FUNCTIONS ================================================*/

T              Nlv_anNew( );
int            Nlv_anProcess( T, Nlv_AData *);
int            Nlv_anAddCalc( T, struct Nlv_cc* );
struct Nlv_cc *Nlv_anGetCalc( T, int calcId );
struct Nlv_cc *Nlv_anGetCalcByName( T, char* nm );
void           Nlv_anSetWindow( T, Nlv_Bounds *b );
Nlv_Bounds    *Nlv_anGetWindow( T );
void           Nlv_anAnalyzeWindow( T, int numClients );
void           Nlv_anAnalyzeWindowContinued( T, int numClients );
void           Nlv_anAnalyzeWindowDone( T );
int            Nlv_anAnalyzeWindowIsStarted( T );
void           Nlv_anClearWindow( T );
void           Nlv_anUpdateWindow( T );
void           Nlv_anSetAnalysisMode( T, int v );
int            Nlv_anGetAnalysisMode( T );

#undef T
#endif
