/****************************************************************** 
 *
 * highlightclass.h
 *
 *     Interface to highlight class.
 *     A list of graph elements which can be added to easily,
 *     cleared, and redrawn.
 *
 * $Author: mstoufer $
 * $Source: /home/g2/src/projects/NetLogger/src/nlv/highlightclass.h,v $
 * $RCSfile: highlightclass.h,v $
 * $Revision: 1.1.1.1 $
 * $Locker:  $
 * $Date: 2000/12/05 19:52:32 $
 *
 *
 ****************************************************************** 
 */
#ifndef NLV_HIGHLIGHT_INCLUDED
#define NLV_HIGHLIGHT_INCLUDED

#include <string.h>
#include <tcl.h>
#include <tk.h>
#include <blt.h>
#include <bltGraph.h>
#include <bltGrElem.h>
#include "graphclass.h"
#include "util.h"
#include "errclass.h"
#include "structs.h"

/*========= DATA =====================================================*/

#define T Nlv_hi
typedef struct T *T;

/*========= FUNCTIONS ================================================*/

T         Nlv_hiNew( );
void      Nlv_hiInit( T );
Element * Nlv_hiGet( T, int idx );
int       Nlv_hiAddCoords( T, double x, double y );
void      Nlv_hiSetSymbol( T, const char *newSymbol );
void      Nlv_hiSetColor( T, int idx, const char *matchColor );
void      Nlv_hiClearAll( T );
void      Nlv_hiDraw( T );

#undef T
#endif
