typedef union {
    int    ival;
    double dval;
    char  *strval;
} YYSTYPE;
#define	TITLE	257
#define	NAME	258
#define	QUOTED_STRING	259
#define	FORMULA	260
#define	HOSTALIAS	261
#define	EVENTSET	262
#define	EVENTS	263
#define	TYPE	264
#define	KSTYPE	265
#define	ID	266
#define	HOSTID	267
#define	CALC	268
#define	ANNOTATE	269
#define	MMCHOICE	270
#define	TIMEORD	271
#define	ROTATECL	272
#define	AUTOSCALE	273
#define	CACTION	274
#define	BOOL	275
#define	INT	276
#define	LIST_START	277
#define	LIST_END	278
#define	VAL	279
#define	FLOAT	280
#define	ANALYSIS	281
#define	EPSILON	282
#define	KEYS	283
#define	BRANGE	284
#define	RLIMIT	285
#define	EOFEXIT	286
#define	FIRSTEVENT	287


extern YYSTYPE nlvkylval;
