/* 
 * nlvAppInit.h
 *
 *	Functions for private app. initialization.
 *
 * $Author: mstoufer $
 * $Source: /home/g2/src/projects/NetLogger/src/nlv/nlvAppInit.h,v $
 * $RCSfile: nlvAppInit.h,v $
 * $Revision: 1.1.1.1 $
 * $Locker:  $
 * $Date: 2000/12/05 19:52:32 $
 */
void Nlv_CopyArgs( int argc, char **argv );
