set f [open fake.log]
set r 10000
set i 0
set sec(0) {}
set line(0) {}
while { ![eof $f] } {
  set ln [gets $f]
  regexp -indices {NL\.SEC=([^ \t]+)} $ln m s
  set line($i) $ln
  set sec($i,s) [lindex $s 0]
  set sec($i,e) [lindex $s 1]
  incr i
}
set n [expr $i-1]
for { set i 0 } { $i < $r } { incr i } {
   for { set j 0 } { $j < $n } { incr j } {
       set pt1 [string range $line($j) 0 [expr $sec($j,s)-1] ]
       set pt2 [expr [string range $line($j) $sec($j,s) $sec($j,e)] + $i]
       set pt3 [string range $line($j) [expr $sec($j,e) + 1] end]
       puts "$pt1$pt2$pt3"
   }
   after 1000
}
