/****************************************************************** 
 *
 * fileclass.h --
 *
 *     Interface to reading and repositioning in a log file.
 *
 * $Author: mstoufer $
 * $Source: /home/g2/src/projects/NetLogger/src/nlv/fileclass.h,v $
 * $RCSfile: fileclass.h,v $
 * $Revision: 1.1.1.1 $
 * $Locker:  $
 * $Date: 2000/12/05 19:52:32 $
 *
 *
 ****************************************************************** 
 */
#ifndef NLV_FILECLASS_INCLUDED
#define NLV_FILECLASS_INCLUDED

#include "structs.h"
#include "errclass.h"

/*========= DATA =====================================================*/

#define T Nlv_fi
typedef struct T *T;

/*========= FUNCTIONS ================================================*/

T       Nlv_fiNew( );
void    Nlv_fiOpen( T, char *name );
int     Nlv_fiClose( T );
LogLine Nlv_fiGetLine( T, LogLine ln );
LogLine Nlv_fiGetLineNoSave( T, LogLine ln );
int     Nlv_fiSeek( T, long pos );
char *  Nlv_fiGetName( T );
int     Nlv_fiGetFd( T );
long    Nlv_fiGetPos( T  );
int     Nlv_fiSeekEOF( T );
int     Nlv_fiGetEOF( T );
long    Nlv_fiGetSize( T );
long    Nlv_fiGetPrevReadPos( T );
int     Nlv_fiLineStartR( T );

#undef T
#endif

