/****************************************************************** 
 *
 * main.h --
 *
 *     Header for main "C" part of NLV.
 *
 *
 * $Author: mstoufer $
 * $Source: /home/g2/src/projects/NetLogger/src/nlv/main.h,v $
 * $RCSfile: main.h,v $
 * $Revision: 1.1.1.1 $
 * $Locker:  $
 * $Date: 2000/12/05 19:52:32 $
 *
 ****************************************************************** 
 */
#ifndef NLV_MAIN_INCLUDED
#define NLV_MAIN_INCLUDED

#include <tcl.h>
#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include <stdlib.h>
#include <pthread.h>
#include <signal.h>
#ifdef solaris
#include <thread.h>
#endif
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "structs.h"
#include "fileclass.h"
#include "indexclass.h"
#include "parseclass.h"
#include "clientsclass.h"
#include "graphclass.h"
#include "keysclass.h"
#include "errclass.h"
#include "util.h"
#include "nlvTkInit.h"


/*
 * Tk file handler
 */
void FileHandler( ClientData clientData,
		  int mask );


#endif

